package controller

import (
	"Medical_OAuth/app/admin/model"
	"Medical_OAuth/app/admin/service"
	"Medical_OAuth/app/admin/service/dto"
	"Medical_OAuth/common/actions"
	"encoding/json"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
	"gogs.baozhida.cn/zoie/OAuth-core/sdk/config"
)

type SysDeptController struct {
	api.Api
}

// GetPage 获取部门列表
// @Summary 获取部门列表
// @Description 获取部门列表
// @Tags 部门
// @Param deptName query string false "部门名称"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysDept}} "{"code": 200, "data": [...]}"
// @Router /api/dept [get]
// @Security Bearer
func (e SysDeptController) GetPage(c *gin.Context) {
	s := service.SysDept{}
	req := dto.SysDeptGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	p.DeptId = user.GetDeptId(c)
	req.DeptId = user.GetDeptId(c)
	list := make([]model.SysDept, 0)
	list, err = s.SetDeptPage(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(list, "查询成功")
}

// Get 通过id获取部门
// @Summary 通过id获取部门
// @Description 通过id获取部门
// @Tags 部门
// @Param id path string true "部门id"
// @Success 200 {object} response.Response{data=model.SysDept} "{"code": 200, "data": [...]}"
// @Router /api/dept/{id} [get]
// @Security Bearer
func (e SysDeptController) Get(c *gin.Context) {
	s := service.SysDept{}
	req := dto.SysDeptGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.SysDept
	//数据权限检查
	err = s.Get(&req, &object)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加部门
// @Summary 添加部门
// @Description 添加部门
// @Tags 部门
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysDeptInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/dept [post]
// @Security Bearer
func (e SysDeptController) Insert(c *gin.Context) {
	s := service.SysDept{}
	req := dto.SysDeptInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	deptId := user.GetDeptId(c)
	err = s.Insert(&req, deptId)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改部门
// @Summary 修改部门
// @Description 修改部门
// @Tags 部门
// @Accept  application/json
// @Product application/json
// @Param id path string true "部门id"
// @Param data body dto.SysDeptUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/dept [put]
// @Security Bearer
func (e SysDeptController) Update(c *gin.Context) {
	s := service.SysDept{}
	req := dto.SysDeptUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除部门
// @Summary 删除部门
// @Description 删除部门
// @Tags 部门
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysDeptDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/dept [delete]
// @Security Bearer
func (e SysDeptController) Delete(c *gin.Context) {
	s := service.SysDept{}
	req := dto.SysDeptDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	var count int64
	err = userSvc.GetCount(&dto.SysUserGetCountReq{DeptIds: []int{req.Id}}, &count)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	if count > 0 {
		e.Error(500, err, "有用户关联，禁止删除！")
		return
	}

	//数据权限检查
	//p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, nil)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}

// Enter 进入部门
// @Summary 删除部门
// @Description 删除部门
// @Tags 部门
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysDeptEnterReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/dept/enter [post]
// @Security Bearer
func (e SysDeptController) Enter(c *gin.Context) {
	s := service.SysDept{}
	req := dto.SysDeptEnterReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	var object model.SysDept
	//数据权限检查
	err = s.Get(&dto.SysDeptGetReq{
		Id: req.Id,
	}, &object)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	err = e.Cache.Set(model.GetEnterDeptCacheKey(c), req.GetId(), int(config.JwtConfig.Timeout))
	err = e.Cache.Set(model.GetEnterDeptNameCacheKey(c), object.DeptName, int(config.JwtConfig.Timeout))
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, "进入失败")
		return
	}
	data := make([]map[string]interface{}, 0)
	err = json.Unmarshal([]byte(model.HaveDeptMenu), &data)
	if err != nil {
		e.Error(500, err, err.Error())
	}
	e.OK(data, "进入成功")
}
