package controller

import (
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"

	"Medical_OAuth/app/admin/model"
	"Medical_OAuth/app/admin/service"
	"Medical_OAuth/app/admin/service/dto"
)

type SysDictTypeController struct {
	api.Api
}

// GetPage 获取字典类型列表
// @Summary 字典类型列表数据
// @Description 字典类型列表数据
// @Tags 字典类型
// @Param dictName query string false "dictName"
// @Param dictId query string false "dictId"
// @Param dictType query string false "dictType"
// @Param pageSize query int false "页条数"
// @Param pageIndex query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysDictType}} "{"code": 200, "data": [...]}"
// @Router /api/dict/type [get]
// @Security Bearer
func (e SysDictTypeController) GetPage(c *gin.Context) {
	s := service.SysDictTypeService{}
	req := dto.SysDictTypeGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Form).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	list := make([]model.SysDictType, 0)
	var count int64
	err = s.GetPage(&req, &list, &count)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取字典类型
// @Summary 通过id获取字典类型
// @Description 通过id获取字典类型
// @Tags 字典类型
// @Param id path int true "字典类型id"
// @Success 200 {object} response.Response{data=model.SysDictType} "{"code": 200, "data": [...]}"
// @Router /api/dict/type/{id} [get]
// @Security Bearer
func (e SysDictTypeController) Get(c *gin.Context) {
	s := service.SysDictTypeService{}
	req := dto.SysDictTypeGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.SysDictType
	err = s.Get(&req, &object)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(object, "查询成功")
}

// Insert 创建字典类型
// @Summary 添加字典类型
// @Description 添加字典类型
// @Tags 字典类型
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysDictTypeInsertReq true "data"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/dict/type [post]
// @Security Bearer
func (e SysDictTypeController) Insert(c *gin.Context) {
	s := service.SysDictTypeService{}
	req := dto.SysDictTypeInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	req.SetCreateBy(user.GetUserId(c))
	err = s.Insert(&req)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改字典类型
// @Summary 修改字典类型
// @Description 修改字典类型
// @Tags 字典类型
// @Accept  application/json
// @Product application/json
// @Param id path int true "字典类型id"
// @Param data body dto.SysDictTypeUpdateReq true "body"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/dict/type/{id} [put]
// @Security Bearer
func (e SysDictTypeController) Update(c *gin.Context) {
	s := service.SysDictTypeService{}
	req := dto.SysDictTypeUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除字典类型
// @Summary 删除字典类型
// @Description 删除字典类型
// @Tags 字典类型
// @Param dictCode body dto.SysDictTypeDeleteReq true "body"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/dict/type [delete]
// @Security Bearer
func (e SysDictTypeController) Delete(c *gin.Context) {
	s := service.SysDictTypeService{}
	req := dto.SysDictTypeDeleteReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	req.SetUpdateBy(user.GetUserId(c))
	err = s.Remove(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}

// GetAll 字典类型查询【代码生成】
// @Summary 字典类型查询【代码生成】
// @Description 字典类型查询【代码生成】
// @Tags 字典类型
// @Param dictName query string false "dictName"
// @Param dictId query string false "dictId"
// @Param dictType query string false "dictType"
// @Success 200 {object} response.Response{data=[]model.SysDictType}  "{"code": 200, "data": [...]}"
// @Router /api/dict/type-option-select [get]
// @Security Bearer
func (e SysDictTypeController) GetAll(c *gin.Context) {
	s := service.SysDictTypeService{}
	req := dto.SysDictTypeGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Form).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	list := make([]model.SysDictType, 0)
	err = s.GetAll(&req, &list)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(list, "查询成功")
}
