package controller

import (
	"Medical_OAuth/app/admin/model"
	"Medical_OAuth/app/admin/service"
	"Medical_OAuth/app/admin/service/dto"
	"Medical_OAuth/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
)

type SysRegisterAuthController struct {
	api.Api
}

// GetPage 获取公开用户注册授权列表
// @Summary 获取公开用户注册授权列表
// @Description 获取公开用户注册授权列表
// @Tags 公开用户注册授权
// @Param postName query string false "公开用户注册授权名称"
// @Param status query int false "状态 1-停用 2-正常"
// @Param sort query int false "排序 ASC-升序 DESC-降序"
// @Param pageSize query int false "页条数"
// @Param pageIndex query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysRegisterAuth}} "{"code": 200, "data": [...]}"
// @Router /api/register-auth [get]
// @Security Bearer
func (e SysRegisterAuthController) GetPage(c *gin.Context) {
	s := service.SysRegisterAuth{}
	req := dto.SysRegisterAuthPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Form).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	list := make([]model.SysRegisterAuth, 0)
	var count int64
	err = s.GetPage(&req, &list, &count)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取公开用户注册授权
// @Summary 通过id获取公开用户注册授权
// @Description 通过id获取公开用户注册授权
// @Tags 公开用户注册授权
// @Param id path int true "公开用户注册授权id"
// @Success 200 {object} response.Response{data=model.SysRegisterAuth} "{"code": 200, "data": [...]}"
// @Router /api/register-auth/{id} [get]
// @Security Bearer
func (e SysRegisterAuthController) Get(c *gin.Context) {
	s := service.SysRegisterAuth{}
	req := dto.SysRegisterAuthGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.SysRegisterAuth
	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Get(&req, &object, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加公开用户注册授权
// @Summary 添加公开用户注册授权
// @Description 添加公开用户注册授权
// @Tags 公开用户注册授权
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysRegisterAuthInsertReq true "data"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/register-auth [post]
// @Security Bearer
func (e SysRegisterAuthController) Insert(c *gin.Context) {
	s := service.SysRegisterAuth{}
	req := dto.SysRegisterAuthInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	req.SetCreateBy(user.GetUserId(c))
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改公开用户注册授权
// @Summary 修改公开用户注册授权
// @Description 修改公开用户注册授权
// @Tags 公开用户注册授权
// @Accept  application/json
// @Product application/json
// @Param id path int true "公开用户注册授权id"
// @Param data body dto.SysRegisterAuthUpdateReq true "body"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/register-auth/{id} [put]
// @Security Bearer
func (e SysRegisterAuthController) Update(c *gin.Context) {
	s := service.SysRegisterAuth{}
	req := dto.SysRegisterAuthUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetUpdateBy(user.GetUserId(c))

	err = s.Update(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除公开用户注册授权
// @Summary 删除公开用户注册授权
// @Description 删除公开用户注册授权
// @Tags 公开用户注册授权
// @Accept  application/json
// @Product application/json
// @Param id body dto.SysRegisterAuthDeleteReq true "请求参数"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/register-auth [delete]
// @Security Bearer
func (e SysRegisterAuthController) Delete(c *gin.Context) {
	s := service.SysRegisterAuth{}
	req := dto.SysRegisterAuthDeleteReq{}

	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Remove(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}
