package controller

import (
	"Medical_OAuth/app/admin/model"
	"Medical_OAuth/app/admin/service"
	"Medical_OAuth/app/admin/service/dto"
	"Medical_OAuth/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	coreModel "gogs.baozhida.cn/zoie/OAuth-core/model"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
	"strconv"
)

type SysService struct {
	api.Api
}

// GetPage 获取服务列表
// @Summary 获取服务列表
// @Description 获取服务列表
// @Tags 服务
// @Param name query string false "服务名称"
// @Param status query int false "状态 1-停用 2-正常"
// @Param pageSize query int false "页条数"
// @Param pageIndex query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysService}} "{"code": 200, "data": [...]}"
// @Router /api/service [get]
// @Security Bearer
func (e SysService) GetPage(c *gin.Context) {
	s := service.SysService{}
	req := dto.SysServicePageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Form).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	list := make([]model.SysService, 0)
	var count int64
	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.GetPage(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取服务
// @Summary 通过id获取服务
// @Description 通过id获取服务
// @Tags 服务
// @Param id path int true "服务id"
// @Success 200 {object} response.Response{data=model.SysService} "{"code": 200, "data": [...]}"
// @Router /api/service/{id} [get]
// @Security Bearer
func (e SysService) Get(c *gin.Context) {
	s := service.SysService{}
	req := dto.SysServiceGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.SysService
	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Get(&req, &object, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加服务
// @Summary 添加服务
// @Description 添加服务
// @Tags 服务
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysServiceInsertReq true "data"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/service [post]
// @Security Bearer
func (e SysService) Insert(c *gin.Context) {
	s := service.SysService{}
	req := dto.SysServiceInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetCreateBy(user.GetUserId(c))
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改服务
// @Summary 修改服务
// @Description 修改服务
// @Tags 服务
// @Accept  application/json
// @Product application/json
// @Param id path int true "服务id"
// @Param data body dto.SysServiceUpdateReq true "body"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/service/{id} [put]
// @Security Bearer
func (e SysService) Update(c *gin.Context) {
	s := service.SysService{}
	req := dto.SysServiceUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetUpdateBy(user.GetUserId(c))

	err = s.Update(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除服务
// @Summary 删除服务
// @Description 删除服务
// @Tags 服务
// @Accept  application/json
// @Product application/json
// @Param id body dto.SysServiceDeleteReq true "请求参数"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/service [delete]
// @Security Bearer
func (e SysService) Delete(c *gin.Context) {
	s := service.SysService{}
	req := dto.SysServiceDeleteReq{}

	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Remove(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}

// Discovery 服务发现
// @Summary 服务发现
// @Description 服务发现，服务主动推送到数据库保存，有则更新，没有则删除。
// @Tags 服务
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysServiceDiscoveryReq true "data"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/service-disco [post]
func (e SysService) Discovery(c *gin.Context) {
	s := service.SysService{}
	req := dto.SysServiceDiscoveryReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	err = s.Discovery(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(req.GetId(), "创建成功")
}

// UserInfo 服务获取用户信息
// @Summary 服务获取用户信息
// @Description 服务获取用户信息，验证登录
// @Tags 服务
// @Accept  application/json
// @Product application/json
// @Success 200 {object} response.Response{data=coreModel.UserInfo} "{"code": 200, "data": [...]}"
// @Router /api/userinfo [get]
func (e SysService) UserInfo(c *gin.Context) {
	s := service.SysService{}
	err := e.MakeContext(c).
		MakeOrm().
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	info := coreModel.UserInfo{
		Uuid:      user.GetUUID(c),
		UserId:    user.GetUserId(c),
		RoleId:    user.GetRoleId(c),
		DeptId:    user.GetDeptId(c),
		UserName:  user.GetUserName(c),
		RoleName:  user.GetRoleName(c),
		RoleKey:   user.GetRoleKey(c),
		DataScope: user.GetDataScope(c),
		DeptName:  user.GetDeptName(c),
	}

	if deptIdStr, err := e.Cache.Get(model.GetEnterDeptCacheKey(c)); err == nil {
		info.DeptId, _ = strconv.Atoi(deptIdStr)
	}
	if deptNameStr, err := e.Cache.Get(model.GetEnterDeptNameCacheKey(c)); err == nil {
		info.DeptName = deptNameStr
	}
	e.Logger.Debug("info", info)

	e.OK(info, "ok")
}
